{-
    Copyright 2011 Saurabh Srivastava (saurabhs@cs.umd.edu)

    This file is part of PINS.

    PINS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PINS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PINS.  If not, see <http://www.gnu.org/licenses/>.

-}
module DomProj
where

import Parser
import Data.List
import DomLibInvert

domproj :: (Map, [Term]) -> [Term]
domproj (m,ts) = transl m $ concatMap termproj ts

transl :: Map -> [Term] -> [Term]
transl m = map (translterm m)

translterm m (Expr e) = Expr (translexpr m e)
translterm m (Pred (Lt e1 e2))  = Pred (Lt (translexpr m e1) (translexpr m e2)) 
translterm m (Pred (Lte e1 e2)) = Pred (Lte (translexpr m e1) (translexpr m e2)) 
translterm m (Pred (Gt e1 e2))  = Pred (Gt (translexpr m e1) (translexpr m e2)) 
translterm m (Pred (Gte e1 e2)) = Pred (Gte (translexpr m e1) (translexpr m e2)) 
translterm m (Pred (Eq e1 e2)) = Pred (Eq (translexpr m e1) (translexpr m e2)) 
translterm m (Pred (Neq e1 e2)) = Pred (Neq (translexpr m e1) (translexpr m e2)) 

translexpr m (Plus e1 e2) = Plus (translexpr m e1) (translexpr m e2)
translexpr m (Minus e1 e2) = Minus (translexpr m e1) (translexpr m e2)
translexpr m (Upd e1 e2 e3) = Upd (translexpr m e1) (translexpr m e2) (translexpr m e3)
translexpr m (Sel e1 e2) = Sel (translexpr m e1) (translexpr m e2)
translexpr m (Atom a) = 
    case (translatom m a) of
        Atom b -> Atom b
        x -> x
translexpr m (Call fn args) = 
    case translatom m fn of
        Atom a -> Call a (map (translexpr m) args)
        x -> error "Function call is calculated expr"

translatom (Map maps) a = 
    case find (\(x,y) -> x == a) maps of
       Nothing -> Atom a 
       Just (x,y) -> y     

termproj e = concatMap (\f -> f e) library

