{-
    Copyright 2011 Saurabh Srivastava (saurabhs@cs.umd.edu)

    This file is part of PINS.

    PINS is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    PINS is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with PINS.  If not, see <http://www.gnu.org/licenses/>.

-}
module PrettyPrint
where

import Parser

prettyprint :: [Term] -> IO ()
prettyprint ts = putStr $ concatMap (\t -> (prettyTerm t) ++ "\n") ts

prettyTerm :: Term -> String
prettyTerm (Expr e) = prettyExpr e
prettyTerm (Pred p) = prettyPred p

prettyPred (Lt e1 e2) = (prettyExpr e1) ++ " < " ++ (prettyExpr e2)
prettyPred (Lte e1 e2) = (prettyExpr e1) ++ " <= " ++ (prettyExpr e2)
prettyPred (Gt e1 e2) = (prettyExpr e1) ++ " > " ++ (prettyExpr e2)
prettyPred (Gte e1 e2) = (prettyExpr e1) ++ " >= " ++ (prettyExpr e2)
prettyPred (Eq e1 e2) = (prettyExpr e1) ++ " = " ++ (prettyExpr e2)
prettyPred (Neq e1 e2) = (prettyExpr e1) ++ " != " ++ (prettyExpr e2)

prettyExpr (Plus e1 e2) = (prettyExpr e1) ++ " + " ++ (prettyExpr e2)
prettyExpr (Minus e1 e2) = (prettyExpr e1) ++ " - " ++ (prettyExpr e2)
prettyExpr (Upd e1 e2 e3) = "upd(" ++ (prettyExpr e1) ++ "," ++ (prettyExpr e2) ++ "," ++ (prettyExpr e3) ++ ")"
prettyExpr (Sel e1 e2) = "sel(" ++ (prettyExpr e1) ++ "," ++ (prettyExpr e2) ++ ")"
prettyExpr (Atom a) = prettyAtom a
prettyExpr (Call fn args) = prettyAtom fn ++ "(" ++ (foldr (\a b->if b=="" then a else a++","++b) "" $ map prettyExpr args) ++ ")"

prettyAtom (Int i) = show i
prettyAtom (Var v) = v
prettyAtom Bot = "##"
